SLASH_NKEYSTONE1 = '/nklist';
SLASH_NKEYSTONE2 = '/nk';

NKeystone = {
	AddonPrefix = "NKEYSTONE",
	KeystoneItemID = 138019,
	RequestInfoTag = "NKEYSTONEREQUESTINFO",
	RequestMaxLevelTag = "NKEYSTONEREQUESTMAXLEVEL"
};

local nkt = NKeystone;

RegisterAddonMessagePrefix(nkt.AddonPrefix);

function SlashCmdList.NKEYSTONE(msg, editbox)
	if msg == "keystones" then
		GetKeystoneInfo();
	elseif msg == "maxlevel" then
		GetMaxLevel();
	else
		print("NKeystone");
		print("  /nk keystones");
		print("  /nk maxlevel");
	end
end

function GetMythicSummary()
	local maxLevel, mapID = GetMythicDetails()
	local mapName = C_ChallengeMode.GetMapInfo(mapID);
	if maxLevel and mapName then
		return "Max level: " .. maxLevel .. " on map: " .. mapName;
	else
		return "No Mythic+ completions during this reset";
	end
end

function GetMythicDetails()
	maps = { };
	C_ChallengeMode.GetMapTable(maps);
	maxLevel = 0;
	mapID = nil;
	for i = 1, #maps do
		local _, _, level = C_ChallengeMode.GetMapPlayerStats(maps[i]);
		if level and level > maxLevel then
			maxLevel = level;
			mapID = maps[i];
		end
	end
	return maxLevel, mapID
end

function GetMaxLevel()
	print("NK: Requesting Max Level");
	SendAddonMessage(nkt.AddonPrefix, nkt.RequestMaxLevelTag, "GUILD");
end

function GetKeystoneInfo()
	print("NK: Requesting Keystone Info");
	SendAddonMessage(nkt.AddonPrefix, nkt.RequestInfoTag, "GUILD");
end

function SendKeystoneInfo(target)
	local bagID, slotID = GetBagAndSlotIDForKeystone();
	local itemLink = GetItemLinkFromBagAndSlotID(bagID, slotID);
	if itemLink ~= nil then
		dist = target and "WHISPER" or "GUILD";
		SendAddonMessage(nkt.AddonPrefix, itemLink, dist, target);
	end
end

function SendMaxLevel(target)
	local mythicSummary = GetMythicSummary();
	dist = target and "WHISPER" or "GUILD";
	SendAddonMessage(nkt.AddonPrefix, mythicSummary, dist, target);
end

function GetBagAndSlotIDForKeystone()
	for bagID = 0, NUM_BAG_SLOTS do
		local slotCount = GetContainerNumSlots(bagID);
		for slotID = 0, slotCount do
			local itemID = GetContainerItemID(bagID, slotID);
			if itemID == nkt.KeystoneItemID then
				return bagID, slotID;
			end
		end
	end
	return nil;
end

function GetItemLinkFromBagAndSlotID(bagID, slotID)
	if bagID == nil or slotID == nil then
		return nil;
	end

	local texture, itemCount, locked, quality, readable, lootable, itemLink
		= GetContainerItemInfo(bagID, slotID);
	return itemLink;
end

function GetItemStringFromItemLink(itemLink)
	return select(3, strfind(itemLink, "|H(.+)|h"));
end

local eventFrame = CreateFrame("FRAME");
eventFrame:RegisterEvent("CHAT_MSG_ADDON");
eventFrame:SetScript("OnEvent", function(...) nkt.OnEvent(...); end);

nkt.OnEvent = function(self, event, ...)
	C_ChallengeMode.RequestMapInfo();
	if event == "CHAT_MSG_ADDON" then
		local prefix, msg, dist, sender = ...;
		if prefix == nkt.AddonPrefix then
			if msg == nkt.RequestInfoTag then
				SendKeystoneInfo(sender);
			elseif msg == nkt.RequestMaxLevelTag then
				SendMaxLevel(sender);
			else
				if dist == "WHISPER" then
					print("NK: " .. sender .. ": " .. msg);
				end
			end
		end
	end
end